#include<cstring>

#include"app.h"
#include"myWindow.h"

#include<Mime.h>
#include<View.h>
#include<Window.h>

#define ATTR_MSG 'attr'

/* global lists */

BList *	allTypesList = new BList(20);
					// every attribute type appearing in the selected files 
					// is put into this list once
BList * activeTypesList = new BList(20);
					// user-selected types to be included in extraction are stored here
BList * inactiveTypesList = new BList(20);
					// types deselected by the user go into this list


/* global variables */

myatfApp * atfApp;

extern myWindow * atfWindow;


int
main()
{
	atfApp = new myatfApp();
	atfApp->Run();
	delete atfApp;
} // main()


extern "C"
void
process_refs(entry_ref dir_ref, BMessage *msg, void *)
{
// from "RunInTerminal" by Kirilla:

	msg->what = B_REFS_RECEIVED; 
	msg->AddRef("dir_ref", & dir_ref); 
	be_roster->Launch("application/x-vnd.fb-atf", msg);
} // process_refs()


myatfApp::myatfApp() : BApplication("application/x-vnd.fb-atf") 
{
	// set defaults
	Outfilepath.SetTo("/boot/home/Desktop/AtF Outfile");
	SetFormat(CSVFORMAT);
}


void
myatfApp::AboutRequested()
{
	BAlert	*aboutAlert;
	aboutAlert = new BAlert ("About", "AttributesToFile Tracker Add-on\nby Finn Bastiansen 2005\nVersion 1.0beta", "Great"); 
	aboutAlert->Go();
}


bool
myatfApp::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return (true);
}


void
myatfApp::RefsReceived( BMessage *msg)
{
	entry_ref dir_ref;
	msg->FindRef("dir_ref", &dir_ref);

	int numberOfRefs = CreateFileList(dir_ref, msg);
	
	if (numberOfRefs != 0) 
	{
		CreateAllTypesList();

		BuildGUI();

		// The rest of the functionality is invoked from window class -- MessageReceived()
	}
	else
	{
		NoRefsAlert();
	} // else
} // RefsReceived()


/* ************* */
/* Own functions */
/* ************* */


void
myatfApp::BuildGUI()
{
		myWindow * atfWindow = new myWindow();
		atfWindow->Show();
}


void
myatfApp::CopyFilenamesToVector()
{
	StringVector.resize( fileList->CountItems() + 1 );

	StringVector[0].push_back( new BString("File") );
	
	for (unsigned int i=1; i < StringVector.size(); i++)
	{
	
		char buffer[B_FILE_NAME_LENGTH];

		( (BEntry *) (fileList->ItemAt(i-1)))->GetName( buffer );

		StringVector[0].push_back( new BString(buffer) );
	} // for (i)

} // myatfApp::CopyFilenamesToVector()


void
myatfApp::CopyAttributesToVector()
{
	BEntry * loopEntry = NULL;
	char loopEntryName[B_FILE_NAME_LENGTH];

	StringVector.resize( activeTypesList->CountItems() + 1 );
	int activeTypesListCounter = activeTypesList->CountItems();

	// iterate through list of attribute types
	for (int i = 0; i < activeTypesListCounter; i++)
	{

		// look at each file if it has an attribute of this type
		// 1. Iterate through fileList.
		int fileListCounter = fileList->CountItems();

		StringVector[i+1].resize(1);
		StringVector[i+1][0] = new BString( ((BString *) (activeTypesList->ItemAt(i)))->String() );


		for (int j = 0; j < fileListCounter ; j++) 
		{
			loopEntry = (BEntry *) fileList->ItemAt(j);
			loopEntry->GetName(loopEntryName);

			if ( !loopEntry->Exists() ) {cout << "Entry does not exist.\n";}
			else 
			{
				// 2. Every item of fileList gets processed. Does this file have an attribute of this type?

				BNode FileToOpen(loopEntry);
				
				// iterate through all attributes of this file
				char buf[B_ATTR_NAME_LENGTH]; 


				bool wasFound = false;
				BString AttrString;

				// loop once through attribute types
				while (FileToOpen.GetNextAttrName(buf) == B_OK) 
				{

					// if file has an attribute of this type
					if ( ((BString *) (activeTypesList->ItemAt(i)))->Compare( buf ) == 0 )
					{
						wasFound = true;

						attr_info attr_type;

						FileToOpen.GetAttrInfo(buf, &attr_type);


						switch (attr_type.type)
							{
							case B_INT8_TYPE:
							case B_UINT8_TYPE:		int8 int8_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &int8_buffer, 
																		sizeof(int8));
													AttrString << int8_buffer;
													break;
					
							case B_INT16_TYPE:
							case B_UINT16_TYPE:		int16 int16_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &int16_buffer, 
																		sizeof(int16));
					
													AttrString << int16_buffer;
													break;
					
							case B_INT32_TYPE:
							case B_UINT32_TYPE:		int32 int32_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &int32_buffer, 
																		sizeof(int32));
													AttrString << int32_buffer;
													break;
					
							case B_INT64_TYPE:
							case B_UINT64_TYPE: 	int64 int64_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &int64_buffer, 
																		sizeof(int64));
													AttrString << int64_buffer;
													break;
						
							case B_BOOL_TYPE:		bool bool_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &bool_buffer, 
																		sizeof(bool));
													if (bool_buffer==true)  AttrString << "true";
													else AttrString << "false";
													break;
					
							case B_FLOAT_TYPE:
							case B_DOUBLE_TYPE:		double double_buffer;
													FileToOpen.ReadAttr(buf, 0, 0, &double_buffer, 
																		sizeof(double));
													AttrString << (float) double_buffer;
													break;
						
							case B_STRING_TYPE:		char char_buffer[255];
													FileToOpen.ReadAttr(buf, B_STRING_TYPE, 0, char_buffer, 
																		sizeof(char_buffer));
													AttrString << char_buffer;
													break;
					
							case B_MIME_TYPE:		char mime_buffer[1024];
													FileToOpen.ReadAttr(buf, B_MIME_TYPE, 0, mime_buffer, 
																		sizeof(mime_buffer));
					
													AttrString << mime_buffer;
													break;
					
							case B_RAW_TYPE:		AttrString.Append("*raw data*\t");
													break;
					
							default:				
													char anything_else_buffer[255];
													FileToOpen.ReadAttr(buf, B_STRING_TYPE, 0, anything_else_buffer, 
																		sizeof(anything_else_buffer));
													AttrString << anything_else_buffer;
													break;	
							} // switch()
					} // if ( Compare() )
					
				} // while (FileToOpen.GetNextAttrName)

				if (wasFound == false)
				{
					AttrString.SetTo( emptyChars.String() );
				}

				StringVector[i+1].push_back( new BString(AttrString.String()) );

			} // else ( i.e. loopEntry->Exists() )

		} // for (j) , i.e. fileListCounter

	} // for (i) , i.e. activeTypesListCounter
	
	loopEntry->Unset();

} // myatfApp::CopyAttributesToVector()


void
myatfApp::CopyVectorToOutfile()
{
	BFile Outfile(Outfilepath.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);

	if (Outfile.InitCheck() != B_OK) 
	{
			BAlert *OutfileInit;
			OutfileInit = new BAlert("outfile_create_fail", 
										"File could not be created",
										"Hm...",
										NULL,
										NULL, 
										B_WIDTH_AS_USUAL,
										B_OFFSET_SPACING, 
										B_WARNING_ALERT);
			OutfileInit->Go();
	}

	else

	{
		Outfile.Write(startChars.String(), startChars.Length());

		int i=0;

		int firstLine;
		if (listFilenames == true) 
		{
			firstLine = 0;
		}
		else
		{
			firstLine = 1;
		}

		for (unsigned int y = 0; y < StringVector[i].size(); y++)
		{
			for (unsigned int x = firstLine; x < StringVector.size(); x++)
			{
				Outfile.Write(StringVector[x][y]->String(), (StringVector[x][y])->Length());
				if (x != StringVector.size() -1) Outfile.Write(separatorChars.String(), separatorChars.Length());
				i++;
			} // for (x)
			if (y != StringVector[y].size() -1) Outfile.Write(lineEndChars.String(), lineEndChars.Length());
			i=0;
		} // for (y)

		Outfile.Write(endChars.String(), endChars.Length());
	
		Outfile.WriteAttr("BEOS:TYPE", B_STRING_TYPE, 0, (void *) (outfileType.String()), 11);
		Outfile.Unset();
		Outfilepath.Unset();
	} // else

} // myatfApp::CopyVectorToOutfile()


void
myatfApp::CopyVectorToOutfile_OneColumnPerFile()
{
	BPath Outfilepath;
	Outfilepath.SetTo("/boot/home/Desktop/AttributeToFile Outfile");
	
	BFile Outfile(Outfilepath.Path(), B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE);

	if (Outfile.InitCheck() != B_OK) 
	{
			BAlert *OutfileInit;
			OutfileInit = new BAlert("outfile_create_fail", 
										"File could not be created",
										"Hm...",
										NULL,
										NULL, 
										B_WIDTH_AS_USUAL,
										B_OFFSET_SPACING, 
										B_WARNING_ALERT);
			OutfileInit->Go();
	}

	else

	{
		Outfile.Write(startChars.String(), startChars.Length());

		// write vector content into the file
		for (unsigned int col = 0; col < StringVector.size(); col++)
		{
			for (unsigned int row = 0; row < StringVector[col].size(); row++)
			{
				Outfile.Write(StringVector[col][row]->String(), (StringVector[col][row])->Length());
				if (row != StringVector[col].size() -1) Outfile.Write(separatorChars.String(), separatorChars.Length());
			} // for (row)
			if (col != StringVector.size() -1) Outfile.Write(lineEndChars.String(), lineEndChars.Length());
		} // for (col)

		Outfile.Write(endChars.String(), endChars.Length());
	
		Outfile.WriteAttr("BEOS:TYPE", B_STRING_TYPE, 0, (void *) (outfileType.String()), 11);
		Outfile.Unset();
		Outfilepath.Unset();
	} // else

} // myatfApp::CopyVectorToOutfile_OneColumnPerFile()


void
myatfApp::CreateAllTypesList()
{
	char typeBuffer[B_ATTR_NAME_LENGTH]; 
	BNode currentFile;
	bool addToList;

	int fileListCounter = fileList->CountItems();

	for (int i = 0; i < fileListCounter; i++)
	{
		BNode currentFile( (BEntry *) fileList->ItemAt(i) );
		while (currentFile.GetNextAttrName(typeBuffer) == B_OK)
		{
			addToList = true;
				if ( IsTypeInList( typeBuffer, allTypesList ) == true )
					{
						addToList = false;
					} // if ( IsTypeInList() )

			if (addToList == true)
			{
				allTypesList->AddItem( (new BString(typeBuffer)) );

			} // if (addToList)
			
		} // while (currentFile)
	} // for (listCounter)
} // myatfApp::CreateAllTypesList()


int
myatfApp::CreateFileList(entry_ref ref, BMessage *msg)
{
	int refcount; 
	BEntry argEntry;
	BPath argPath;

	fileList = new BList(20); 

	for (refcount = 0; msg->FindRef("refs", refcount, &ref) == B_NO_ERROR; refcount++) 
	{ 
		argEntry.SetTo(&ref);
		argEntry.GetPath(&argPath);		
		
		if ( argEntry.IsFile() )
		{
			BEntry * listEntry = new BEntry(argEntry);
			fileList->AddItem(listEntry);
			
		} // if (argEntry.IsFile())
		else if ( argEntry.IsDirectory() )
		{
			BDirectory argDir(&ref);
					
			/* get dir entries; T O D O : whole function should be recursive someday to handle sub-sub-dirs, too */
			BEntry dirEntry;
			while (argDir.GetNextEntry(&dirEntry, false) != B_ENTRY_NOT_FOUND)
			{
				BEntry * listEntry = new BEntry(dirEntry);
				fileList->AddItem(listEntry);
			} // while (argDir)

		} // else if ( argEntry.IsDirectory() )

	} // for (refcount)

	return refcount;
} // myatfApp::CreateFileList()


bool
myatfApp::IsTypeInList(char * buffer, BList * list)
{
	bool isInList = false;

	if ( list->CountItems() != 0)
	{
		for (int listCounter = 0; ( listCounter < list->CountItems() ) && ( isInList==false ); listCounter++)
		{
			if (list->ItemAt(listCounter) != NULL)
			{
				if ( strcmp( buffer, ( ((BString *) list->ItemAt(listCounter))->String() ) ) == 0 )
				{
					isInList = true;
				}
			}
		} // for
	} // if
	
	return (isInList);
} // myatfApp::IsTypeInList()


void
myatfApp::Help()
{
	char * args[] = {"/boot/home/config/documentation/AttributesToFile documentation/AttributesToFile.html"};
	be_roster->Launch("application/x-vnd.Be-NPOS", 1, args); // Open Helpfile in Net+
} // myatfApp::Help()


void
myatfApp::NoRefsAlert()
{
	BAlert *noarg_alert = new BAlert("noarg_alert", "No parameters were passed to the app. What do you want to do?", 
										"Quit", "I need help!", NULL, B_WIDTH_AS_USUAL, 
										B_OFFSET_SPACING, B_WARNING_ALERT); 	
	
	noarg_alert->SetShortcut(0, B_ESCAPE);
	int32 noarg_alert_result = noarg_alert->Go();
	
	switch(noarg_alert_result)
	{
		case 0: 
				break;
		case 1: Help(); 
				break;
	}// switch
} // myatfApp::NoRefsAlert()


void
myatfApp::SetFormat(int format)
{
	switch (format)
	{
		case 1: // CSV
			startChars.SetTo("");
			separatorChars.SetTo("\t");
			emptyChars.SetTo(" ");
			lineEndChars.SetTo("\n");
			endChars.SetTo("");

			outfileType.SetTo("text/plain");
			break;

		case 2: // HTML	
			startChars.SetTo("");
			startChars << "<HTML>\n<HEAD>\n<TITLE>" << Outfilepath.Path() << "</TITLE>\n</HEAD>\n<BODY>\n<TABLE BORDER=1>\n<TR><TD>";
			separatorChars.SetTo("</TD><TD>");
			emptyChars.SetTo("&nbsp;");
			lineEndChars.SetTo("</TD></TR>\n<TR><TD>");
			endChars.SetTo("</TD></TR>\n</BODY>\n</HTML>");

			outfileType.SetTo("text/html");
			break;

	} // switch(format)
}
